@@ -2,13 +2,14 @@ from typing import Callable, Optional, Sequence, Type, TypeVar, cast
 
 from django.db.models.base import Model
 import strawberry
-from strawberry.arguments import UNSET
+from strawberry import UNSET
 from strawberry.field import StrawberryField
 from strawberry.utils.typing import __dataclass_transform__
 from strawberry_django.fields.field import field as _field
-from strawberry_django.fields.types import is_auto
 from strawberry_django.ordering import Ordering
 
+from strawberry_django_plus.utils.typing import is_auto
+
 from . import field
 from .relay import connection, node
 
@@ -38,14 +39,15 @@ def order(  # noqa:A001
         for fname, type_ in cls.__annotations__.items():
             if is_auto(type_):
                 type_ = Ordering
-            cls.__annotations__[fname] = Optional[type_]
+
+            cls.__annotations__[fname] = Optional[type_]  # type:ignore
             setattr(cls, fname, UNSET)
 
         return strawberry.input(
             cls,
             name=cast(str, name),
             description=cast(str, description),
-            directives=directives,  # type:ignore
+            directives=directives,
         )
 
     return wrapper
