@@ -36,8 +36,9 @@ import uuid
 
 from graphql import GraphQLID
 import strawberry
+from strawberry import UNSET
 from strawberry.annotation import StrawberryAnnotation
-from strawberry.arguments import UNSET, StrawberryArgument
+from strawberry.arguments import StrawberryArgument
 from strawberry.custom_scalar import ScalarDefinition
 from strawberry.field import StrawberryField
 from strawberry.lazy_type import LazyType
@@ -73,7 +74,7 @@ __all__ = [
 
 _T = TypeVar("_T")
 _R = TypeVar("_R")
-_connection_typename = "arrayconnection"
+connection_typename = "arrayconnection"
 
 NodeType = TypeVar("NodeType", bound="Node")
 
@@ -94,7 +95,7 @@ def from_base64(value: str) -> Tuple[str, str]:
 
     """
     try:
-        res = base64.b64decode(value.encode()).decode().split(":")
+        res = base64.b64decode(value.encode()).decode().split(":", 1)
     except Exception as e:
         raise ValueError(str(e)) from e
 
@@ -330,9 +331,14 @@ class Node(abc.ABC):
     All types that are relay ready should inherit from this interface and
     implement the following methods.
 
+    Attributes:
+        id_attr:
+            (Optional) Define id field of node
+
     Methods:
         resolve_id:
-            (Optional) Called to resolve the node's id. By default ir returns `node.id`
+            (Optional) Called to resolve the node's id.
+            By default it returns `getattr(node, getattr(node, 'id_attr'. 'id'))`
             to use the one provided when creating the node itself.
         resolve_node:
             Called to retrieve a node given its id
@@ -383,7 +389,8 @@ class Node(abc.ABC):
     ) -> AwaitableOrValue[str]:
         """Resolve the node id.
 
-        By default this returns `node.id`. Override this to return something else.
+        By default this returns `getattr(node, getattr(node, 'id_attr'. 'id'))`.
+        Override this to return something else.
 
         Args:
             info:
@@ -395,7 +402,8 @@ class Node(abc.ABC):
             The resolved id (which is expected to be str)
 
         """
-        return root.id
+        id_attr = getattr(cls, "id_attr", "id")
+        return getattr(root, id_attr)
 
     @classmethod
     def resolve_connection(
@@ -615,8 +623,9 @@ class Connection(Generic[NodeType]):
     edges: List[Edge[NodeType]] = strawberry.field(
         description="Contains the nodes in this connection",
     )
-    total_count: int = strawberry.field(
+    total_count: Optional[int] = strawberry.field(
         description="Total quantity of existing nodes",
+        default=None,
     )
 
     @classmethod
@@ -660,38 +669,36 @@ class Connection(Generic[NodeType]):
 
         """
         if total_count is None:
+            # Support ORMs that define .count() (e.g. django)
             try:
-                # Support ORMs that define .count() (e.g. django)
-                total_count = cast(int, nodes.count())  # type:ignore
-            except (AttributeError, TypeError):
+                total_count = int(nodes.count())  # type:ignore
+            except (AttributeError, ValueError, TypeError):
                 if isinstance(nodes, Sized):
                     total_count = len(nodes)
-                else:
-                    nodes = list(nodes)
-                    total_count = len(nodes)
 
-        start = 0
-        end = total_count
-
-        if after:
-            after_type, after_parsed = from_base64(after)
-            assert after_type == _connection_typename
-            start = max(start, int(after_parsed) + 1)
-        if before:
-            before_type, before_parsed = from_base64(before)
-            assert before_type == _connection_typename
-            end = min(end, int(before_parsed))
-
-        has_first_or_last = False
         max_results = config.RELAY_MAX_RESULTS
         if max_results is None:
             max_results = math.inf
 
+        start = 0
+        end = total_count if total_count is not None else math.inf
+
+        if after:
+            after_type, after_parsed = from_base64(after)
+            assert after_type == connection_typename
+            start = int(after_parsed) + 1
+        if before:
+            before_type, before_parsed = from_base64(before)
+            assert before_type == connection_typename
+            end = int(before_parsed)
+
+        if end is None:
+            end = max_results
+
         if isinstance(first, int):
             if first < 0:
                 raise ValueError("Argument 'first' must be a non-negative integer.")
 
-            has_first_or_last = True
             if first > max_results:
                 raise ValueError(f"Argument 'first' cannot be higher than {max_results}.")
 
@@ -703,21 +710,40 @@ class Connection(Generic[NodeType]):
             if last > max_results:
                 raise ValueError(f"Argument 'last' cannot be higher than {max_results}.")
 
-            has_first_or_last = True
+            if end == math.inf:
+                raise ValueError("Cannot use last with unlimited results")
+
             start = max(start, end - last)
 
-        if not has_first_or_last:
-            end = min(end, start + max_results)
+        # If at this point end is still inf, consider it to be start + max_results
+        if end == math.inf:
+            end = start + max_results
 
+        expected = end - start
+        # If no parameters are given, end could be total_results at this point.
+        # Make sure we don't exceed max_results in here
+        if expected > max_results:
+            end = start + max_results
+            expected = end - start
+
+        # Overfetch by 1 to check if we have a next result
         edges = [
-            Edge(cursor=to_base64(_connection_typename, start + i), node=v)
-            for i, v in enumerate(cast(Sequence, nodes)[start:end])
+            Edge(cursor=to_base64(connection_typename, start + i), node=v)
+            for i, v in enumerate(cast(Sequence, nodes)[start : end + 1])  # noqa:E203
         ]
+
+        # Remove the overfetched result
+        if len(edges) == expected + 1:
+            edges = edges[:-1]
+            has_next_page = True
+        else:
+            has_next_page = False
+
         page_info = PageInfo(
             start_cursor=edges[0].cursor if edges else None,
             end_cursor=edges[-1].cursor if edges else None,
             has_previous_page=start > 0,
-            has_next_page=end < total_count,
+            has_next_page=has_next_page,
         )
 
         return cls(
@@ -1002,7 +1028,13 @@ class InputMutationField(RelayField):
                 name=arg.graphql_name,
                 is_subscription=arg.is_subscription,
                 description=arg.description,
-                default=arg.default,
+                # FIXME: We are using default_factory instead of default to allow the default
+                # value to be UNSET if it was given as such.
+                default_factory=(
+                    (lambda _arg=arg: _arg.default)
+                    if arg.default is not dataclasses.MISSING
+                    else UNSET
+                ),
                 directives=directives,
             )
             arg_field.graphql_name = arg.graphql_name
@@ -1028,10 +1060,7 @@ class InputMutationField(RelayField):
 
     @property
     def arguments(self) -> List[StrawberryArgument]:
-        args = [
-            arg for arg in super().arguments if arg.python_name in StrawberryResolver._SPECIAL_ARGS
-        ]
-        return args + list(self.default_args.values())
+        return list(self.default_args.values())
 
     def get_result(
         self,
