@@ -19,24 +19,20 @@ from django.core.exceptions import FieldDoesNotExist
 from django.db.models.base import Model
 from django.db.models.fields.reverse_related import ManyToManyRel, ManyToOneRel
 import strawberry
-from strawberry import UNSET, auto
+from strawberry import UNSET
 from strawberry.annotation import StrawberryAnnotation
 from strawberry.field import StrawberryField
 from strawberry.types.fields.resolver import StrawberryResolver
 from strawberry.unset import UnsetType
 from strawberry.utils.typing import __dataclass_transform__
 from strawberry_django.fields.field import field as _field
-from strawberry_django.fields.types import (
-    get_model_field,
-    is_auto,
-    resolve_model_field_name,
-)
+from strawberry_django.fields.types import get_model_field, resolve_model_field_name
 from strawberry_django.type import StrawberryDjangoType as _StraberryDjangoType
 from strawberry_django.utils import get_annotations, is_similar_django_type
 from typing_extensions import Annotated
 
 from strawberry_django_plus.optimizer import OptimizerStore, PrefetchType
-from strawberry_django_plus.utils.typing import TypeOrSequence
+from strawberry_django_plus.utils.typing import TypeOrSequence, is_auto
 
 from . import field
 from .descriptors import ModelProperty
@@ -229,7 +225,7 @@ def _get_fields(django_type: "StrawberryDjangoType"):
         if name in fields:
             continue
 
-        attr = getattr(origin, name)
+        attr = getattr(origin, name, None)
         if not isinstance(attr, StrawberryField):
             continue
 
@@ -253,6 +249,7 @@ def _process_type(
     cls: _O,
     model: Type[Model],
     *,
+    field_cls: Type[StrawberryDjangoField] = StrawberryDjangoField,
     filters: Optional[type] = UNSET,
     order: Optional[type] = UNSET,
     pagination: Optional[bool] = UNSET,
@@ -267,6 +264,7 @@ def _process_type(
     django_type = StrawberryDjangoType(
         origin=cls,
         model=model,
+        field_cls=field_cls,
         is_input=kwargs.get("is_input", False),
         is_partial=kwargs.pop("partial", False),
         is_filter=kwargs.pop("is_filter", False),
@@ -288,7 +286,7 @@ def _process_type(
     for f in fields:
         annotation = f.type_annotation.annotation if f.type_annotation is not None else f.type
         if annotation is None:
-            annotation = StrawberryAnnotation(auto)
+            annotation = StrawberryAnnotation(strawberry.auto)
 
         cls.__annotations__[f.name] = annotation
         setattr(cls, f.name, f)
@@ -324,7 +322,7 @@ def _process_type(
 
         if not _has_own_node_resolver(cls, "resolve_id"):
             cls.resolve_id = types.MethodType(
-                lambda cls, root, *args, **kwargs: resolve_model_id(root),
+                lambda cls, root, *args, **kwargs: resolve_model_id(cls, root),
                 cls,
             )
 
@@ -369,6 +367,7 @@ def type(  # noqa:A001
     model: Type[Model],
     *,
     name: Optional[str] = None,
+    field_cls: Type[StrawberryDjangoField] = StrawberryDjangoField,
     is_input: bool = False,
     is_interface: bool = False,
     is_filter: Union[Literal["lookups"], bool] = False,
@@ -400,6 +399,7 @@ def type(  # noqa:A001
             cls,
             model,
             name=name,
+            field_cls=field_cls,
             is_input=is_input,
             is_filter=is_filter,
             is_interface=is_interface,
@@ -434,6 +434,7 @@ def interface(
     model: Type[Model],
     *,
     name: Optional[str] = None,
+    field_cls: Type[StrawberryDjangoField] = StrawberryDjangoField,
     description: Optional[str] = None,
     directives: Optional[Sequence[object]] = (),
 ) -> Callable[[_T], _T]:
@@ -454,6 +455,7 @@ def interface(
             cls,
             model,
             name=name,
+            field_cls=field_cls,
             is_interface=True,
             description=description,
             directives=directives,
@@ -478,6 +480,7 @@ def input(  # noqa:A001
     model: Type[Model],
     *,
     name: Optional[str] = None,
+    field_cls: Type[StrawberryDjangoField] = StrawberryDjangoField,
     description: Optional[str] = None,
     directives: Optional[Sequence[object]] = (),
     is_filter: Union[Literal["lookups"], bool] = False,
@@ -500,6 +503,7 @@ def input(  # noqa:A001
             cls,
             model,
             name=name,
+            field_cls=field_cls,
             is_input=True,
             is_filter=is_filter,
             description=description,
@@ -526,6 +530,7 @@ def partial(  # noqa:A001
     model: Type[Model],
     *,
     name: Optional[str] = None,
+    field_cls: Type[StrawberryDjangoField] = StrawberryDjangoField,
     description: Optional[str] = None,
     directives: Optional[Sequence[object]] = (),
 ) -> Callable[[_T], _T]:
@@ -546,6 +551,7 @@ def partial(  # noqa:A001
             cls,
             model,
             name=name,
+            field_cls=field_cls,
             is_input=True,
             description=description,
             directives=directives,
